// Copyright (C) 2005 Universitat d'Alacant / Universidad de Alicante
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of the
// License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, see <https://www.gnu.org/licenses/>.

#ifndef PRETRANSFER_H
#define PRETRANSFER_H

#include <unicode/ustdio.h>
#include <lttoolbox/input_file.h>
#include <lttoolbox/ustring.h>

UString storeAndWriteWblank(InputFile& input, UFILE *output);
void readAndWriteUntil(InputFile& input, UFILE *output, int const charcode);
void procWord(InputFile& input, UFILE *output, bool surface_forms, bool compound_sep, UString wblank);
void processStream(InputFile& input, UFILE *output, bool null_flush, bool surface_forms, bool compound_sep);

#endif
