#ifndef __TRANSFER_REGEX_OPTIMIZER__
#define __TRANSFER_REGEX_OPTIMIZER__

#include <lttoolbox/ustring.h>
#include <vector>

UString optimize_regex(const std::vector<UString>& options);

#endif // __TRANSFER_REGEX_OPTIMIZER__
